@@ -1,37 +1,36 @@
 /************************************************************************/
 /*                                                                      */
-/* Psionic PortSentry - Port Scan Detector				*/
+/* Psionic PortSentry							*/
 /*                                                                      */
-/* Author: Craig H. Rowland <crowland@psionic.com>                      */
-/* Created: 10-12-97                                                    */
-/* Modified: 04-12-99                                                   */
+/* Created: 10-12-1997                                                  */
+/* Modified: 06-26-2001                                                 */
 /*                                                                      */
-/* Send all changes/modifications/bugfixes to the above address.        */
+/* Send all changes/modifications/bugfixes to help@psionic.com      	*/
 /*                                                                      */
 /*                                                                      */
-/* This software is Copyright(c) 1997-99 Craig H. Rowland               */
+/* This software is Copyright(c) 1997-2001 Psionic Software, Inc.       */
 /*                                                                      */
 /* Disclaimer:                                                          */
 /*                                                                      */
-/* All software distributed by Craig H. Rowland ("the author") and      */
-/* Psionic Systems is distributed AS IS and carries NO WARRANTY or      */
-/* GUARANTEE OF ANY KIND. End users of the software acknowledge that    */
-/* they will not hold the author, Psionic Systems, and any employer of  */
-/* the author liable for failure or non-function of a software          */
-/* product. YOU ARE USING THIS PRODUCT AT YOUR OWN RISK                 */
+/* All software distributed by Psionic Software is distributed 		*/
+/* AS IS and carries NO WARRANTY or GUARANTEE OF ANY KIND. End users of */
+/* the software acknowledge that they will not hold Psionic Software	*/
+/* liable for failure or non-function of the software product. YOU ARE 	*/
+/* USING THIS PRODUCT AT YOUR OWN RISK.					*/
 /*                                                                      */
-/* Licensing restrictions apply. See the license that came with this    */
-/* file for more information or visit http://www.psionic.com for more   */
-/* information.                                                         */
+/* Licensing restrictions apply. Commercial re-sell is prohibited under */
+/* certain conditions. See the license that came with this package or 	*/
+/* visit http://www.psionic.com for more information. 			*/
 /*                                                                      */
-/*$Id: portsentry.c,v 1.26 1999/08/04 05:26:27 crowland Exp crowland $ */
+/* $Id: portsentry.c,v 1.35 2001/06/26 15:14:27 crowland Exp crowland $ */
 /************************************************************************/
 
+
 #include "portsentry.h"
 #include "portsentry_io.h"
 #include "portsentry_util.h"
 
-/* Global variables...ugghhh.... */
+/* Global variables */
 char gblScanDetectHost[MAXSTATE][IPMAXBUF];
 char gblKillRoute[MAXBUF];
 char gblKillHostsDeny[MAXBUF];
@@ -44,9 +43,10 @@ char gblDetectionType[MAXBUF];
 int gblScanDetectCount = 0;
 int gblBlockTCP = 0;
 int gblBlockUDP = 0;
+int gblRunCmdFirst = 0;
+int gblResolveHost = 0;
 int gblConfigTriggerCount = 0;
 
-
 int
 main (int argc, char *argv[])
 {
@@ -175,8 +175,9 @@ InitConfig (void)
   FILE *input;
   char configToken[MAXBUF];
 
-  gblBlockTCP = DoBlockTCP ();
-  gblBlockUDP = DoBlockUDP ();
+  gblBlockTCP = CheckFlag ("BLOCK_TCP");
+  gblBlockUDP = CheckFlag ("BLOCK_UDP");
+  gblResolveHost = CheckFlag ("RESOLVE_HOST");
 
   memset (gblKillRoute, '\0', MAXBUF);
   memset (gblKillHostsDeny, '\0', MAXBUF);
@@ -230,6 +231,9 @@ InitConfig (void)
       Log ("debug: InitConfig: retrieved KILL_RUN_CMD option: %s \n",
 	   gblKillRunCmd);
 #endif
+	/* Check the order we should run the KILL_RUN_CMD */
+	/* Default is to run the command after blocking */
+	gblRunCmdFirst = CheckFlag ("KILL_RUN_CMD_FIRST");
     }
   else
     {
@@ -326,7 +330,7 @@ PacketReadTCP (int socket, struct iphdr *ipPtr, struct tcphdr *tcpPtr)
   if ((ipPtr->ihl < 5) || (ipPtr->ihl > 15))
     {
       addr.s_addr = (u_int) ipPtr->saddr;
-      Log ("attackalert: Illegal IP header length detected: %d from (possible) host: %s\n",
+      Log ("attackalert: Illegal IP header length detected in TCP packet: %d from (possible) host: %s\n",
 	 ipPtr->ihl, inet_ntoa (addr));
       return (FALSE);
     }
@@ -358,7 +362,7 @@ PacketReadUDP (int socket, struct iphdr *ipPtr, struct udphdr *udpPtr)
   if ((ipPtr->ihl < 5) || (ipPtr->ihl > 15))
     {
       addr.s_addr = (u_int) ipPtr->saddr;
-      Log ("attackalert: Illegal IP header length detected: %d from (possible) host: %s\n",
+      Log ("attackalert: Illegal IP header length detected in UDP packet: %d from (possible) host: %s\n",
 	 ipPtr->ihl, inet_ntoa (addr));
       return (FALSE);
     }
@@ -387,7 +391,6 @@ PortSentryStealthModeTCP (void)
   int openSockfd = 0, incomingPort = 0;
   char *temp, target[IPMAXBUF], configToken[MAXBUF];
   char resolvedHost[DNSMAXBUF], *packetType;
-  struct hostent *hostPtr = NULL;
   struct in_addr addr;
   struct iphdr ip;
   struct tcphdr tcp;
@@ -495,14 +498,19 @@ PortSentryStealthModeTCP (void)
 		      scanDetectTrigger = CheckStateEngine (target);
 		      if (scanDetectTrigger == TRUE)
 			{
-			hostPtr = gethostbyaddr ((char *) &ip.saddr,
-				sizeof (ip.saddr), AF_INET);
-			if (hostPtr != NULL)
-		    		snprintf (resolvedHost, DNSMAXBUF, "%s",
-			     		hostPtr->h_name);
+			if (gblResolveHost) /* Do they want DNS resolution? */
+			{
+				if(CleanAndResolve(resolvedHost, target) != TRUE)
+				{
+		      			Log ("attackalert: ERROR: Error resolving host. \
+					      resolving disabled for this host.\n");
+					snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+				}			
+			}
 			else
+			{
 				snprintf (resolvedHost, DNSMAXBUF, "%s", target);
-
+			}
 
 			  packetType = ReportPacketType (tcp);
 			  Log ("attackalert: %s from host: %s/%s to TCP port: %d",
@@ -558,7 +566,6 @@ PortSentryAdvancedStealthModeTCP (void)
   unsigned int count = 0, inUsePorts[MAXSOCKS], portCount = 0;
   char target[IPMAXBUF], configToken[MAXBUF];
   char resolvedHost[DNSMAXBUF], *temp, *packetType;
-  struct hostent *hostPtr;
   struct in_addr addr;
   struct iphdr ip;
   struct tcphdr tcp;
@@ -642,7 +649,7 @@ PortSentryAdvancedStealthModeTCP (void)
 	  for (count = 0; count < portCount; count++)
 	    {
 	      if ((incomingPort == inUsePorts[count])
-		  || (incomingPort > advancedPorts))
+		  || (incomingPort >= advancedPorts))
 		{
 		  hotPort = FALSE;
 		  break;
@@ -673,15 +680,21 @@ PortSentryAdvancedStealthModeTCP (void)
 		      /* check if they've visited before */
 		      scanDetectTrigger = CheckStateEngine (target);
 
-		      if (scanDetectTrigger == TRUE)
+			if (scanDetectTrigger == TRUE)
 			{
-			  hostPtr = gethostbyaddr ((char *) &ip.saddr,
-					   sizeof (ip.saddr), AF_INET);
-			  if (hostPtr != NULL)
-			    snprintf (resolvedHost, DNSMAXBUF, "%s",
-				      hostPtr->h_name);
-			  else
-			    snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+				if (gblResolveHost) /* Do they want DNS resolution? */
+				{
+					if(CleanAndResolve(resolvedHost, target) != TRUE)
+					{
+		      				Log ("attackalert: ERROR: Error resolving host. \
+					      	resolving disabled for this host.\n");
+						snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+					}			
+				}
+				else
+				{
+					snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+				}
 
 			  packetType = ReportPacketType (tcp);
 			  Log ("attackalert: %s from host: %s/%s to TCP port: %d",
@@ -734,7 +747,6 @@ PortSentryStealthModeUDP (void)
   int openSockfd = 0, incomingPort = 0;
   char *temp, target[IPMAXBUF], configToken[MAXBUF];
   char resolvedHost[DNSMAXBUF];
-  struct hostent *hostPtr;
   struct in_addr addr;
   struct iphdr ip;
   struct udphdr udp;
@@ -835,13 +847,19 @@ PortSentryStealthModeUDP (void)
 		  scanDetectTrigger = CheckStateEngine (target);
 		  if (scanDetectTrigger == TRUE)
 		    {
-		      hostPtr = gethostbyaddr ((char *) &ip.saddr,
-				sizeof (ip.saddr), AF_INET);
-		      if (hostPtr != NULL)
-			snprintf (resolvedHost, DNSMAXBUF, "%s",
-				  hostPtr->h_name);
-		      else
-			snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			if (gblResolveHost) /* Do they want DNS resolution? */
+			{
+				if(CleanAndResolve(resolvedHost, target) != TRUE)
+				{
+		      			Log ("attackalert: ERROR: Error resolving host. \
+					resolving disabled for this host.\n");
+					snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+				}			
+			}
+			else
+			{
+				snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			}
 
 		      Log ("attackalert: UDP scan from host: %s/%s to UDP port: %d",
 			 resolvedHost, target, ports2[count]);
@@ -894,7 +912,6 @@ PortSentryAdvancedStealthModeUDP (void)
   unsigned int count = 0, inUsePorts[MAXSOCKS], portCount = 0;
   char target[IPMAXBUF], configToken[MAXBUF];
   char resolvedHost[DNSMAXBUF], *temp;
-  struct hostent *hostPtr;
   struct in_addr addr;
   struct iphdr ip;
   struct udphdr udp;
@@ -975,7 +992,7 @@ PortSentryAdvancedStealthModeUDP (void)
       for (count = 0; count < portCount; count++)
 	{
 	  if ((incomingPort == inUsePorts[count])
-	      || (incomingPort > advancedPorts))
+	      || (incomingPort >= advancedPorts))
 	    {
 	      hotPort = FALSE;
 	      break;
@@ -1009,13 +1026,19 @@ PortSentryAdvancedStealthModeUDP (void)
 
 		  if (scanDetectTrigger == TRUE)
 		    {
-		      hostPtr = gethostbyaddr ((char *) &ip.saddr,
-				       sizeof (ip.saddr), AF_INET);
-		      if (hostPtr != NULL)
-			snprintf (resolvedHost, DNSMAXBUF, "%s",
-				  hostPtr->h_name);
-		      else
-			snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			if (gblResolveHost) /* Do they want DNS resolution? */
+			{
+				if(CleanAndResolve(resolvedHost, target) != TRUE)
+				{
+		      			Log ("attackalert: ERROR: Error resolving host. \
+					resolving disabled for this host.\n");
+					snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+				}			
+			}
+			else
+			{
+				snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			}
 
 		      Log ("attackalert: UDP scan from host: %s/%s to UDP port: %d",
 			 resolvedHost, target, incomingPort);
@@ -1065,12 +1088,11 @@ PortSentryModeTCP (void)
   struct sockaddr_in client, server;
   int length, portCount = 0, ports[MAXSOCKS];
   int openSockfd[MAXSOCKS], incomingSockfd, result = TRUE;
-  int count = 0, scanDetectTrigger = TRUE, showBanner = FALSE;
-  int selectResult = 0, gotBound = FALSE;
+  int count = 0, scanDetectTrigger = TRUE, showBanner = FALSE, boundPortCount = 0;
+  int selectResult = 0;
   char *temp, target[IPMAXBUF], bannerBuffer[MAXBUF], configToken[MAXBUF];
   char resolvedHost[DNSMAXBUF];
   fd_set selectFds;
-  struct hostent *hostPtr;
 
   if ((ConfigTokenRetrieve ("TCP_PORTS", configToken)) == FALSE)
     {
@@ -1112,24 +1134,25 @@ PortSentryModeTCP (void)
     {
       Log ("adminalert: Going into listen mode on TCP port: %d\n",
 	   ports[count]);
-      if ((openSockfd[count] = OpenTCPSocket ()) == ERROR)
+      if ((openSockfd[boundPortCount] = OpenTCPSocket ()) == ERROR)
 	{
 	  Log ("adminalert: ERROR: could not open TCP socket. Aborting.\n");
 	  return (ERROR);
 	}
 
-      if (BindSocket (openSockfd[count], client, server, ports[count]) ==
+      if (BindSocket (openSockfd[boundPortCount], client, server, ports[count]) ==
 	  ERROR)
 	{
 	  Log ("adminalert: ERROR: could not bind TCP socket: %d. Attempting to continue\n",
 	     ports[count]);
 	}
       else			/* well we at least bound to one socket so we'll continue */
-	gotBound = TRUE;
+	  boundPortCount++;
     }
 
+
   /* if we didn't bind to anything then abort */
-  if (gotBound == FALSE)
+  if (boundPortCount == 0)
     {
       Log ("adminalert: ERROR: could not bind ANY TCP sockets. Shutting down.\n");
       return (ERROR);
@@ -1143,7 +1166,7 @@ PortSentryModeTCP (void)
   for (;;)
     {
       /* set up select call */
-      for (count = 0; count < portCount; count++)
+      for (count = 0; count < boundPortCount; count++)
 	FD_SET (openSockfd[count], &selectFds);
       selectResult =
 	select (MAXSOCKS, &selectFds, NULL, NULL, (struct timeval *) NULL);
@@ -1164,7 +1187,7 @@ PortSentryModeTCP (void)
       /* select is reporting a waiting socket. Poll them all to find out which */
       else if (selectResult > 0)
 	{
-	  for (count = 0; count < portCount; count++)
+	  for (count = 0; count < boundPortCount; count++)
 	    {
 	      if (FD_ISSET (openSockfd[count], &selectFds))
 		{
@@ -1203,13 +1226,19 @@ PortSentryModeTCP (void)
 				   strlen (bannerBuffer));
 			  /* we don't need the bonehead anymore */
 			  close (incomingSockfd);
-			  hostPtr = gethostbyaddr ((char *) &client.sin_addr.s_addr,
-				    sizeof (client.sin_addr.s_addr), AF_INET);
-			  if (hostPtr != NULL)
-			    snprintf (resolvedHost, DNSMAXBUF, "%s",
-				      hostPtr->h_name);
+			  if (gblResolveHost) /* Do they want DNS resolution? */
+			  {
+				if(CleanAndResolve(resolvedHost, target) != TRUE)
+				{
+		      			Log ("attackalert: ERROR: Error resolving host. \
+					resolving disabled for this host.\n");
+					snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+				}			
+			  }
 			  else
-			    snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			  {
+				snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			  }
 
 			  Log ("attackalert: Connect from host: %s/%s to TCP port: %d",
 			     resolvedHost, target, ports[count]);
@@ -1259,12 +1288,11 @@ PortSentryModeUDP (void)
   struct sockaddr_in client, server;
   int length, ports[MAXSOCKS], openSockfd[MAXSOCKS], result = TRUE;
   int count = 0, portCount = 0, selectResult = 0, scanDetectTrigger = 0;
-  int gotBound = FALSE, showBanner = FALSE;
+  int boundPortCount = 0, showBanner = FALSE;
   char *temp, target[IPMAXBUF], bannerBuffer[MAXBUF], configToken[MAXBUF];
   char buffer[MAXBUF];
   char resolvedHost[DNSMAXBUF];
   fd_set selectFds;
-  struct hostent *hostPtr;
 
   if ((ConfigTokenRetrieve ("UDP_PORTS", configToken)) == FALSE)
     {
@@ -1305,22 +1333,22 @@ PortSentryModeUDP (void)
     {
       Log ("adminalert: Going into listen mode on UDP port: %d\n",
 	   ports[count]);
-      if ((openSockfd[count] = OpenUDPSocket ()) == ERROR)
+      if ((openSockfd[boundPortCount] = OpenUDPSocket ()) == ERROR)
 	{
 	  Log ("adminalert: ERROR: could not open UDP socket. Aborting\n");
 	  return (ERROR);
 	}
-      if (BindSocket (openSockfd[count], client, server, ports[count]) == ERROR)
+      if (BindSocket (openSockfd[boundPortCount], client, server, ports[count]) == ERROR)
 	{
 	  Log ("adminalert: ERROR: could not bind UDP socket: %d. Attempting to continue\n",
 	     ports[count]);
 	}
       else			/* well we at least bound to one socket so we'll continue */
-	gotBound = TRUE;
+	boundPortCount++;
     }
 
 /* if we didn't bind to anything then abort */
-  if (gotBound == FALSE)
+  if (boundPortCount == 0)
     {
       Log ("adminalert: ERROR: could not bind ANY UDP sockets. Shutting down.\n");
       return (ERROR);
@@ -1334,7 +1362,7 @@ PortSentryModeUDP (void)
   for (;;)
     {
       /* set up select call */
-      for (count = 0; count < portCount; count++)
+      for (count = 0; count < boundPortCount; count++)
 	FD_SET (openSockfd[count], &selectFds);
       /* setup the select multiplexing (blocking mode) */
       selectResult =
@@ -1395,13 +1423,19 @@ PortSentryModeUDP (void)
 				    strlen (bannerBuffer), 0,
 				    (struct sockaddr *) &client, length);
 
-			  hostPtr = gethostbyaddr ((char *) &client.sin_addr.s_addr,
-			  	    sizeof (client.sin_addr.s_addr), AF_INET);
-			  if (hostPtr != NULL)
-			    snprintf (resolvedHost, DNSMAXBUF, "%s",
-				      hostPtr->h_name);
+			  if (gblResolveHost) /* Do they want DNS resolution? */
+			  {
+				if(CleanAndResolve(resolvedHost, target) != TRUE)
+				{
+		      			Log ("attackalert: ERROR: Error resolving host. \
+					resolving disabled for this host.\n");
+					snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+				}			
+			  }
 			  else
-			    snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			  {
+				snprintf (resolvedHost, DNSMAXBUF, "%s", target);
+			  }
 
 			  Log
 			    ("attackalert: Connect from host: %s/%s to UDP port: %d",
@@ -1443,6 +1477,7 @@ DisposeTCP (char *target, int port)
   Log ("debug: DisposeTCP: disposing of host %s on port %d with option: %d",
        target, port, gblBlockTCP);
   Log ("debug: DisposeTCP: killRunCmd: %s", gblKillRunCmd);
+  Log ("debug: DisposeTCP: gblRunCmdFirst: %d", gblRunCmdFirst);
   Log ("debug: DisposeTCP: killHostsDeny: %s", gblKillHostsDeny);
   Log ("debug: DisposeTCP: killRoute: %s  %d", gblKillRoute,
        strlen (gblKillRoute));
@@ -1451,21 +1486,37 @@ DisposeTCP (char *target, int port)
   if (gblBlockTCP == 1)
     {
       /* run external command first, hosts.deny second, dead route last */
-      if (strlen (gblKillRunCmd) > 0)
-	if (KillRunCmd (target, port, gblKillRunCmd) != TRUE)
-	  status = FALSE;
-      if (strlen (gblKillHostsDeny) > 0)
-	if (KillHostsDeny (target, port, gblKillHostsDeny) != TRUE)
-	  status = FALSE;
-      if (strlen (gblKillRoute) > 0)
-	if (KillRoute (target, port, gblKillRoute) != TRUE)
-	  status = FALSE;
+      if (gblRunCmdFirst)
+	{
+      	if (strlen (gblKillRunCmd) > 0)
+		if (KillRunCmd (target, port, gblKillRunCmd, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillHostsDeny) > 0)
+		if (KillHostsDeny (target, port, gblKillHostsDeny, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillRoute) > 0)
+		if (KillRoute (target, port, gblKillRoute, gblDetectionType) != TRUE)
+	  		status = FALSE;
+	}
+      /* run hosts.deny first, dead route second, external command last */
+      else
+	{
+      	if (strlen (gblKillHostsDeny) > 0)
+		if (KillHostsDeny (target, port, gblKillHostsDeny, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillRoute) > 0)
+		if (KillRoute (target, port, gblKillRoute, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillRunCmd) > 0)
+		if (KillRunCmd (target, port, gblKillRunCmd, gblDetectionType) != TRUE)
+	  		status = FALSE;
+	}
     }
   else if (gblBlockTCP == 2)
     {
       /* run external command only */
       if (strlen (gblKillRunCmd) > 0)
-	if (KillRunCmd (target, port, gblKillRunCmd) != TRUE)
+	if (KillRunCmd (target, port, gblKillRunCmd, gblDetectionType) != TRUE)
 	  status = FALSE;
     }
   else
@@ -1481,30 +1532,50 @@ DisposeUDP (char *target, int port)
 {
   int status = TRUE;
 
-
 #ifdef DEBUG
   Log ("debug: DisposeUDP: disposing of host %s on port %d with option: %d",
        target, port, gblBlockUDP);
+  Log ("debug: DisposeUDP: killRunCmd: %d", gblKillRunCmd);
+  Log ("debug: DisposeUDP: gblRunCmdFirst: %s", gblRunCmdFirst);
+  Log ("debug: DisposeUDP: killHostsDeny: %s", gblKillHostsDeny);
+  Log ("debug: DisposeUDP: killRoute: %s  %d", gblKillRoute,
+       strlen (gblKillRoute));
 #endif
 /* Should we ignore TCP from active response? */
   if (gblBlockUDP == 1)
-    {
+    {	
       /* run external command first, hosts.deny second, dead route last */
-      if (strlen (gblKillRunCmd) > 0)
-	if (KillRunCmd (target, port, gblKillRunCmd) != TRUE)
-	  status = FALSE;
-      if (strlen (gblKillHostsDeny) > 0)
-	if (KillHostsDeny (target, port, gblKillHostsDeny) != TRUE)
-	  status = FALSE;
-      if (strlen (gblKillRoute) > 0)
-	if (KillRoute (target, port, gblKillRoute) != TRUE)
-	  status = FALSE;
+      if (gblRunCmdFirst)
+	{
+      	if (strlen (gblKillRunCmd) > 0)
+		if (KillRunCmd (target, port, gblKillRunCmd, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillHostsDeny) > 0)
+		if (KillHostsDeny (target, port, gblKillHostsDeny, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillRoute) > 0)
+		if (KillRoute (target, port, gblKillRoute, gblDetectionType) != TRUE)
+	  		status = FALSE;
+	}
+      /* run hosts.deny first, dead route second, external command last */
+      else
+	{
+      	if (strlen (gblKillHostsDeny) > 0)
+		if (KillHostsDeny (target, port, gblKillHostsDeny, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillRoute) > 0)
+		if (KillRoute (target, port, gblKillRoute, gblDetectionType) != TRUE)
+	  		status = FALSE;
+      	if (strlen (gblKillRunCmd) > 0)
+		if (KillRunCmd (target, port, gblKillRunCmd, gblDetectionType) != TRUE)
+	  		status = FALSE;
+	}
     }
   else if (gblBlockUDP == 2)
     {
       /* run external command only */
       if (strlen (gblKillRunCmd) > 0)
-	if (KillRunCmd (target, port, gblKillRunCmd) != TRUE)
+	if (KillRunCmd (target, port, gblKillRunCmd, gblDetectionType) != TRUE)
 	  status = FALSE;
     }
   else
@@ -1519,7 +1590,7 @@ void
 Usage (void)
 {
   printf ("Psionic PortSentry - Port Scan Detector.\n");
-  printf ("Copyright 1997,98 Craig H. Rowland <crowland@psionic.com>\n");
+  printf ("Copyright 1997-2000 Psionic Software, Inc. help@psionic.com>\n");
   printf ("Licensing restrictions apply. Please see documentation\n");
   printf ("Version: %s\n\n", VERSION);
 #ifdef SUPPORT_STEALTH
@@ -1540,7 +1611,7 @@ CheckStateEngine (char *target)
   int count = 0, scanDetectTrigger = TRUE;
   int gotOne = 0;
 
-/* This is the rather dumb scan state engine. It maintains     */
+/* This is the rather basic scan state engine. It maintains     */
 /* an array of past hosts who triggered a connection on a port */
 /* when a new host arrives it is compared against the array */
 /* if it is found in the array it increments a state counter by */
@@ -1548,7 +1619,7 @@ CheckStateEngine (char *target)
 /* the end is reached or the trigger value has been exceeded */
 /* This would probably be better as a linked list/hash table, */
 /* but for the number of hosts we are tracking this is just as good. */
-/* I'll probably change this in the future */
+/* This will probably change in the future */
 
   gotOne = 1;			/* our flag counter if we get a match */
   scanDetectTrigger = TRUE;	/* set to TRUE until set otherwise */
@@ -1582,9 +1653,9 @@ CheckStateEngine (char *target)
       /* if our array is still less than MAXSTATE large add it to the end */
       if (gblScanDetectCount < MAXSTATE)
 	{
-	  gblScanDetectCount++;
 	  SafeStrncpy (gblScanDetectHost[gblScanDetectCount], target,
 		       IPMAXBUF);
+	  gblScanDetectCount++;
 	}
       else
 	{
@@ -1592,6 +1663,7 @@ CheckStateEngine (char *target)
 	  gblScanDetectCount = 0;
 	  SafeStrncpy (gblScanDetectHost[gblScanDetectCount], target,
 		       IPMAXBUF);
+	  gblScanDetectCount++;
 	}
 
 #ifdef DEBUG
@@ -1628,18 +1700,18 @@ ReportPacketType (struct tcphdr tcpPkt)
 
   if ((tcpPkt.syn == 0) && (tcpPkt.fin == 0) && (tcpPkt.ack == 0) \
       && (tcpPkt.psh == 0) && (tcpPkt.rst == 0) && (tcpPkt.urg == 0))
-    snprintf (packetDesc, MAXBUF, "NULL scan");
+    snprintf (packetDesc, MAXBUF, " TCP NULL scan");
   else if ((tcpPkt.fin == 1) && (tcpPkt.urg == 1) && (tcpPkt.psh == 1))
-    snprintf (packetDesc, MAXBUF, "XMAS scan");
+    snprintf (packetDesc, MAXBUF, "TCP XMAS scan");
   else if ((tcpPkt.fin == 1) && (tcpPkt.syn != 1) && (tcpPkt.ack != 1) \
 	   &&(tcpPkt.psh != 1) && (tcpPkt.rst != 1) && (tcpPkt.urg != 1))
-    snprintf (packetDesc, MAXBUF, "FIN scan");
+    snprintf (packetDesc, MAXBUF, "TCP FIN scan");
   else if ((tcpPkt.syn == 1) && (tcpPkt.fin != 1) && (tcpPkt.ack != 1) \
 	   &&(tcpPkt.psh != 1) && (tcpPkt.rst != 1) && (tcpPkt.urg != 1))
-    snprintf (packetDesc, MAXBUF, "SYN/Normal scan");
+    snprintf (packetDesc, MAXBUF, "TCP SYN/Normal scan");
   else
     snprintf (packetDesc, MAXBUF,
-	      "Unknown Type: Packet Flags: SYN: %d FIN: %d ACK: %d PSH: %d URG: %d RST: %d", 
+	      "Unknown Type: TCP Packet Flags: SYN: %d FIN: %d ACK: %d PSH: %d URG: %d RST: %d", 
 	      tcpPkt.syn, tcpPkt.fin, tcpPkt.ack, tcpPkt.psh, tcpPkt.urg,
 	      tcpPkt.rst);
 
@@ -1653,9 +1725,9 @@ SmartVerifyTCP (struct sockaddr_in client, struct sockaddr_in server,
 
   int testSockfd;
 
-/* Ok here is where I "Smart-Verify" the socket. If the port was previously */
+/* Ok here is where we "Smart-Verify" the socket. If the port was previously */
 /* unbound, but now appears to have someone there, then we will skip responding */
-/* to this inbound packet. This essentially is a (lame) "stateful" inspection of the */
+/* to this inbound packet. This a basic "stateful" inspection of the */
 /* the connection */
 
   if ((testSockfd = OpenTCPSocket ()) == ERROR)
@@ -1685,15 +1757,14 @@ SmartVerifyUDP (struct sockaddr_in client, struct sockaddr_in server,
 {
   int testSockfd;
 
-
-/* Ok here is where I "Smart-Verify" the socket. If the port was previously */
+/* Ok here is where we "Smart-Verify" the socket. If the port was previously */
 /* unbound, but now appears to have someone there, then we will skip responding */
 /* to this inbound packet. This essentially is a "stateful" inspection of the */
 /* the connection */
 
   if ((testSockfd = OpenUDPSocket ()) == ERROR)
     {
-      Log ("adminalert: ERROR: could not open TCP socket to smart-verify.\n");
+      Log ("adminalert: ERROR: could not open UDP socket to smart-verify.\n");
       return (FALSE);
     }
   else
