@@ -1,33 +1,31 @@
 /************************************************************************/
 /*                                                                      */
-/* Sentry - Port Scan Detector IO Library                               */
+/* Psionic PortSentry							*/
 /*                                                                      */
-/* Author: Craig H. Rowland <crowland@psionic.com>                      */
-/*                          <crowland@bipolar.net>                      */
-/* Created: 10-12-97                                                    */
-/* Modified: 03-04-99                                                   */
+/* Created: 10-12-1997                                                  */
+/* Modified: 06-26-2001                                                 */
 /*                                                                      */
-/* Send all changes/modifications/bugfixes to the above addresses.      */
+/* Send all changes/modifications/bugfixes to help@psionic.com      	*/
 /*                                                                      */
 /*                                                                      */
-/* This software is Copyright(c) 1997-99 Craig H. Rowland               */
+/* This software is Copyright(c) 1997-2001 Psionic Software, Inc.       */
 /*                                                                      */
 /* Disclaimer:                                                          */
 /*                                                                      */
-/* All software distributed by Craig H. Rowland ("the author") and      */
-/* Psionic Systems is distributed AS IS and carries NO WARRANTY or      */
-/* GUARANTEE OF ANY KIND. End users of the software acknowledge that    */
-/* they will not hold the author, Psionic Systems, and any employer of  */
-/* the author liable for failure or non-function of a software          */
-/* product. YOU ARE USING THIS PRODUCT AT YOUR OWN RISK                 */
+/* All software distributed by Psionic Software is distributed 		*/
+/* AS IS and carries NO WARRANTY or GUARANTEE OF ANY KIND. End users of */
+/* the software acknowledge that they will not hold Psionic Software	*/
+/* liable for failure or non-function of the software product. YOU ARE 	*/
+/* USING THIS PRODUCT AT YOUR OWN RISK.					*/
 /*                                                                      */
-/* Licensing restrictions apply. See the license that came with this    */
-/* file for more information or visit http://www.psionic.com for more   */
-/* information.                                                         */
+/* Licensing restrictions apply. Commercial re-sell is prohibited under */
+/* certain conditions. See the license that came with this package or 	*/
+/* visit http://www.psionic.com for more information. 			*/
 /*                                                                      */
-/* Id: $Id: portsentry_io.c,v 1.25 1999/11/14 22:37:46 crowland Exp crowland $ */
+/* $Id: portsentry_io.c,v 1.32 2001/06/26 15:14:52 crowland Exp crowland $ */
 /************************************************************************/
 
+
 #include "portsentry.h"
 #include "portsentry_io.h"
 #include "portsentry_util.h"
@@ -104,14 +102,45 @@ DaemonSeed (void)
 }
 
 
+/* Compares an IP address against a listed address and its netmask*/
+int
+CompareIPs(char *target, char *ignoreAddr, int ignoreNetmaskBits)
+{
+  unsigned long int netmaskAddr, ipAddr, targetAddr;
+
+  ipAddr = inet_addr(ignoreAddr);
+  targetAddr = inet_addr(target);
+  netmaskAddr = htonl (0xFFFFFFFF << (32 - ignoreNetmaskBits)); 
+
+
+#ifdef DEBUG
+	Log ("debug: target %s\n", target);
+	Log ("debug: ignoreAddr %s\n", ignoreAddr);
+	Log ("debug: ignoreNetmaskBits %d\n", ignoreNetmaskBits);
+	Log ("debug: ipAddr %lu\n", ipAddr);
+	Log ("debug: targetAddr %lu\n", targetAddr);
+	Log ("debug: netmask %x\n", netmaskAddr);
+	Log ("debug: mix ipAddr %lu\n", (ipAddr & netmaskAddr));
+	Log ("debug: mix target %lu\n", (targetAddr & netmaskAddr));
+#endif
+
+  /* Network portion mask & op and return */
+  if ((ipAddr & netmaskAddr) == (targetAddr & netmaskAddr))
+	return(TRUE);
+  else
+	return(FALSE);
+}
+
+
 
 /* check hosts that should never be blocked */
 int 
 NeverBlock (char *target, char *filename)
 {
   FILE *input;
-  char buffer[MAXBUF], tempBuffer[MAXBUF];
-  int count;
+  char buffer[MAXBUF], tempBuffer[MAXBUF], netmaskBuffer[MAXBUF];
+  char *slashPos;
+  int count = 0, dest = 0, netmaskBits = 0;
 
 #ifdef DEBUG
   Log ("debug: NeverBlock: Opening ignore file: %s \n", filename);
@@ -125,32 +154,60 @@ NeverBlock (char *target, char *filename)
 
   while (fgets (buffer, MAXBUF, input) != NULL)
   {
-	if(strstr(buffer, target) != (char) NULL)
+	/* Reset destination counter */
+	dest = 0;
+
+	if ((buffer[0] == '#') || (buffer[0] == '\n'))
+		continue;
+
+	for(count = 0; count < strlen(buffer); count++)
 	{
-		for(count = 0; count < strlen(buffer); count++)
-		{
-			if((isdigit(buffer[count])) || 
-				(buffer[count] == '.'))
-			{
-				tempBuffer[count] = buffer[count];
-			}
-			else
-			{
-				tempBuffer[count] = '\0';
-				break;
-			}
-		}
-		if(strcmp(target, tempBuffer) == 0)
-		{
-#ifdef DEBUG
-	  		Log ("debug: NeverBlock: Host: %s found in ignore file\n", target);
-#endif
-	  		fclose (input);
-	  		return (TRUE);
-		}
+		/* Parse out digits, colons, and slashes. Everything else rejected */
+        	if((isdigit(buffer[count])) ||
+           	   (buffer[count] == '.') || (buffer[count] == ':') || (buffer[count] == '/'))
+        	{
+          		tempBuffer[dest++] = buffer[count];
+        	}
+        	else
+        	{
+          		tempBuffer[dest] = '\0';
+          		break;
+        	}
+	}
+	
+	/* Return pointer to slash if it exists and copy data to buffer */
+	slashPos = strchr(tempBuffer, '/');
+	if (slashPos)
+	{
+		SafeStrncpy(netmaskBuffer, slashPos + 1, MAXBUF);
+		/* Terminate tempBuffer string at delimeter for later use */
+		*slashPos = '\0';
+	}
+	else
+		/* Copy in a 32 bit netmask if none given */
+		SafeStrncpy(netmaskBuffer, "32", MAXBUF);
+
+
+	/* Convert netmaskBuffer to bits in netmask */
+	netmaskBits = atoi(netmaskBuffer);
+	if ((netmaskBits < 0) || (netmaskBits > 32))
+	{
+		Log ("adminalert: Invalid netmask in config file: %s  Ignoring entry.\n", buffer);
+		continue;
 	}
 
-    }
+	if (CompareIPs(target, tempBuffer, netmaskBits))
+	{
+#ifdef DEBUG
+	  		Log ("debug: NeverBlock: Host: %s found in ignore file with netmask %s\n", target, netmaskBuffer);
+#endif
+	
+	  		fclose (input);
+	  		return (TRUE);
+	}
+
+   } /* end while() */
+
 #ifdef DEBUG
   Log ("debug: NeverBlock: Host: %s NOT found in ignore file\n", target);
 #endif
@@ -268,8 +325,10 @@ ConfigTokenRetrieve (char *token, char *configToken)
 #ifdef DEBUG
 	      Log ("debug: ConfigTokenRetrieve: data: %s", buffer);
 #endif
-	      /* search for the token */
-	      if (strstr (buffer, token) != (char) NULL)
+	      /* search for the token and make sure the trailing character */
+	      /* is a " " or "=" to make sure the entire token was found */
+	      if ((strstr (buffer, token) != (char) NULL) && 
+		   ((buffer[strlen(token)] == '=') || (buffer[strlen(token)] == ' '))) 
 		{		/* cut off the '=' and send it back */
 		  if (strstr (buffer, "\"") == (char) NULL)
 		    {
@@ -414,9 +473,10 @@ OpenRAWUDPSocket (void)
 /* This will use a system() call to change the route of the target host to */
 /* a dead IP address on your LOCAL SUBNET. */
 int 
-KillRoute (char *target, int port, char *killString)
+KillRoute (char *target, int port, char *killString, char *detectionType)
 {
-  char cleanAddr[MAXBUF], commandStringTemp[MAXBUF], commandStringFinal[MAXBUF];
+  char cleanAddr[MAXBUF], commandStringTemp[MAXBUF];
+  char commandStringTemp2[MAXBUF],commandStringFinal[MAXBUF];
   char portString[MAXBUF];
   int killStatus = ERROR, substStatus = ERROR;
 
@@ -434,13 +494,20 @@ KillRoute (char *target, int port, char *killString)
       Log ("adminalert: Error trying to parse $TARGET$ Token for KILL_ROUTE. Skipping.\n");
       return (ERROR);
     }
-  SubstString (portString, "$PORT$", commandStringTemp, commandStringFinal);
-  if (substStatus == ERROR)
+
+  if(SubstString (portString, "$PORT$", commandStringTemp, commandStringTemp2) == ERROR)
     {
       Log ("adminalert: Error trying to parse $PORT$ Token for KILL_ROUTE. Skipping.\n");
       return (ERROR);
     }
 
+  if(SubstString (detectionType, "$MODE$", commandStringTemp2, commandStringFinal) == ERROR)
+    {
+      Log ("adminalert: Error trying to parse $MODE$ Token for KILL_ROUTE. Skipping.\n");
+      return (ERROR);
+    }
+
+
 #ifdef DEBUG
   Log ("debug: KillRoute: running route command: %s\n", commandStringFinal);
 #endif
@@ -470,28 +537,36 @@ KillRoute (char *target, int port, char *killString)
 
 /* This will run a specified command with TARGET as the option if one is given. */
 int 
-KillRunCmd (char *target, int port, char *killString)
+KillRunCmd (char *target, int port, char *killString, char *detectionType)
 {
-  char cleanAddr[MAXBUF], commandStringTemp[MAXBUF], commandStringFinal[MAXBUF];
+  char cleanAddr[MAXBUF], commandStringTemp[MAXBUF];
+  char commandStringTemp2[MAXBUF], commandStringFinal[MAXBUF];
   char portString[MAXBUF];
-  int killStatus = ERROR, substStatus = ERROR;
+  int killStatus = ERROR;
 
   CleanIpAddr (cleanAddr, target);
   snprintf(portString, MAXBUF, "%d", port);
 
-  substStatus = SubstString (cleanAddr, "$TARGET$", killString, commandStringTemp);
-  if (substStatus == ERROR)
+  /* Tokens are not required, but we check for an error anyway */
+  if(SubstString (cleanAddr, "$TARGET$", killString, commandStringTemp) == ERROR)
     {
       Log ("adminalert: Error trying to parse $TARGET$ Token for KILL_RUN_CMD. Skipping.\n");
       return (ERROR);
     }
-  SubstString (portString, "$PORT$", commandStringTemp, commandStringFinal);
-  if (substStatus == ERROR)
+
+  if(SubstString (portString, "$PORT$", commandStringTemp, commandStringTemp2) == ERROR)
     {
       Log ("adminalert: Error trying to parse $PORT$ Token for KILL_RUN_CMD. Skipping.\n");
       return (ERROR);
     }
 
+  if(SubstString (detectionType, "$MODE$", commandStringTemp2, commandStringFinal) == ERROR)
+    {
+      Log ("adminalert: Error trying to parse $MODE$ Token for KILL_RUN_CMD. Skipping.\n");
+      return (ERROR);
+    }
+
+
   /* Kill the bastard and report a status */
   killStatus = system (commandStringFinal);
 
@@ -519,11 +594,12 @@ KillRunCmd (char *target, int port, char *killString)
 /* all access. The drop route metod is preferred as this stops UDP attacks as well */
 /* as TCP. You may find though that host.deny will be a more permanent home.. */
 int 
-KillHostsDeny (char *target, int port, char *killString)
+KillHostsDeny (char *target, int port, char *killString, char *detectionType)
 {
 
   FILE *output;
-  char cleanAddr[MAXBUF], commandStringTemp[MAXBUF], commandStringFinal[MAXBUF];
+  char cleanAddr[MAXBUF], commandStringTemp[MAXBUF];
+  char commandStringTemp2[MAXBUF], commandStringFinal[MAXBUF];
   char portString[MAXBUF];
   int substStatus = ERROR;
 
@@ -547,13 +623,18 @@ KillHostsDeny (char *target, int port, char *killString)
       return (ERROR);
     }
 
-  SubstString (portString, "$PORT$", commandStringTemp, commandStringFinal);
-  if (substStatus == ERROR)
+  if(SubstString (portString, "$PORT$", commandStringTemp, commandStringTemp2) == ERROR)
     {
       Log ("adminalert: Error trying to parse $PORT$ Token for KILL_HOSTS_DENY. Skipping.\n");
       return (ERROR);
     }
 
+  if(SubstString (detectionType, "$MODE$", commandStringTemp2, commandStringFinal) == ERROR)
+    {
+      Log ("adminalert: Error trying to parse $MODE$ Token for KILL_HOSTS_DENY. Skipping.\n");
+      return (ERROR);
+    }
+
 #ifdef DEBUG
   Log ("debug: KillHostsDeny: result string for block: %s\n", commandStringFinal);
 #endif
@@ -693,58 +774,27 @@ return(numberOfSubst);
 }
 
 
-/* This function will check to see if we should ignore UDP scans and not block */
-/* them. This prevents DOS attacks */
+
+/* This function checks a config variable for a numerical flag and returns it */
 int 
-DoBlockUDP (void)
+CheckFlag (char *flagName)
 {
   char configToken[MAXBUF];
 
-  if ((ConfigTokenRetrieve ("BLOCK_UDP", configToken)) == TRUE)
+  if ((ConfigTokenRetrieve (flagName, configToken)) == TRUE)
     {
 #ifdef DEBUG
-      Log ("debug: DoBlockUDP: found BLOCK_UDP string.\n");
+      Log ("debug: CheckFlag: found %s string.\n", flagName);
 #endif
+      return (atoi(configToken));
     }
   else
     {
 #ifdef DEBUG
-      Log ("debug: DoBlockUDP: BLOCK_UDP option not found. Assuming TRUE.");
+      Log ("debug: CheckFlag: %s option not found. Assuming FALSE.\n", flagName);
 #endif
       return (FALSE);
     }
-
-  if ((atoi(configToken) == 1) || (atoi(configToken) == 2))
-    return (atoi(configToken));
-  else
-    return (FALSE);
-}
-
-
-/* This function will check to see if we should ignore TCP */
-int 
-DoBlockTCP (void)
-{
-  char configToken[MAXBUF];
-
-  if ((ConfigTokenRetrieve ("BLOCK_TCP", configToken)) == TRUE)
-    {
-#ifdef DEBUG
-      Log ("debug: DoBlockTCP: found BLOCK_TCP string.\n");
-#endif
-    }
-  else
-    {
-#ifdef DEBUG
-      Log ("debug: DoBlockTCP: BLOCK_TCP option not found. Assuming FALSE.");
-#endif
-      return (FALSE);
-    }
-
-  if ((atoi(configToken) == 1) || (atoi(configToken) == 2))
-    return (atoi(configToken));
-  else
-    return (FALSE);
 }
 
 
@@ -777,3 +827,4 @@ snprintf (char *str, size_t n, char const *fmt,...)
   return (f._ptr - str);
 }
 #endif
+
